==> musiccontrol-0.2.tar.gz: musiccontrol-0.2/src/dk/borderworlds/ModPlayer.java <==
// $Id: ModPlayer.java,v 1.8 2003/12/28 01:18:49 xi Exp $

/******************************************************************************
*    MusicControl - An advanced music player
*    Copyright (C) 2002,2003  Christian Laursen <xi@borderworlds.dk>
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

package dk.borderworlds;

import java.io.*;

public class ModPlayer extends MusicPlayer implements Runnable {
	private Song song;
	private Process proc;
	private BufferedInputStream bis;
	String name;
	private Thread inputreader;
	private String status;
	private byte[] statusLine;
	private String title, filename;
	private int pausePosition;
	private MusicConfiguration configuration;
	private boolean paused;

	public ModPlayer (MusicConfiguration configuration) {
		song = null;
		proc = null;
		this.configuration = configuration;
		paused = false;
	}

	public void playSong (Song song) {
		this.song = song;
		playSong(0);
	}

	public void playSong (int offset) {
		if (offset == 0) {
			status = "Starting";
		} else {
			status = "Resuming";
		}
		title = song.getFilename();
		filename = song.getFilename();
		Runtime rt = Runtime.getRuntime();
		if (proc != null) {
			proc.destroy();
			while (bis != null) {
				try {
					Thread.sleep(300);
				} catch (Exception e) {
				}
			}
			try {
				Thread.sleep(300);
			} catch (Exception e) {
			}
			proc = null;
		}
		try {
			String device = configuration.getDevice();
			String host = configuration.getHost();
			int args = (pausePosition == 0) ? 2 : 4;
			if (device.equals("ESD")) {
				args += 2;
			}
			String[] cmdarray = new String[args];
			cmdarray[0] = "simplemod";
			if (device.equals("ESD")) {
				cmdarray[1] = "-e";
				cmdarray[2] = host;
			}
			if (pausePosition != 0) {
				cmdarray[args-3] = "-k";
				cmdarray[args-2] = "" + pausePosition;
			}
			cmdarray[args - 1] = song.getDirectory() + File.separator + song.getFilename();
			proc = rt.exec(cmdarray);
			InputStream is = proc.getErrorStream();
			bis = new BufferedInputStream(is);
			inputreader = new Thread (this);
			inputreader.start();
			status = "Playing";
		} catch (Exception e) {
			System.err.println(e);
		}
	}

	public void pauseSong () {
		if (statusLine != null) {
			int length = 4;
			while ((statusLine[5+length] == 32) && length > 0) {
				length--;
			}
			if (length == 0) {
				pausePosition = 0;
			} else {
				String secs = new String(statusLine, 6, length);
				pausePosition = Integer.parseInt(secs);
			}
		} else {
			pausePosition = 0;
		}

		paused = true;

		if (proc != null) {
			proc.destroy();
			while (bis != null) {
				try {
					Thread.sleep(300);
				} catch (Exception e) {
				}
			}
			proc = null;
		}
	}

	public void resumeSong () {
		if (paused) {
			playSong(pausePosition);
			pausePosition = 0;
			paused = false;
		}
	}

	public void stopSong () {
		if (proc != null) {
			proc.destroy();
			while (bis != null) {
				try {
					Thread.sleep(300);
				} catch (Exception e) {
				}
			}
			proc = null;
			song = null;
		}
		try {
			Thread.sleep(300);
		} catch (Exception e) {
		}
	}

	public String getStatus () {
		if (status.equals("Retrying")) {
			playSong(0);
			return "Retrying";
		}
		if (status.equals("Playing") && bis == null) {
			if (!paused) {
				status = "Stopped";
				song = null;
			} else {
				status = "Paused";
			}
		}
		return status;
	}

	public String getFilename () {
		if (song == null) {
			return "(none)";
		} else {
			return filename;
		}
	}

	public String getTitle () {
		if (song == null) {
			return "(none)";
		} else {
			return title;
		}
	}

	public String getPosition () {
		if (statusLine != null) {
			int start = 0;
			int length = 5;
			while (statusLine[start] == ' ' && length > 0) {
				start++;
				length--;
			}
			if (length == 0) {
				return("N/A");
			}
			String secs = new String(statusLine, start, length);
			int seconds = 0;
			try {
				seconds = Integer.parseInt(secs);
			} catch (Exception e) {
			}
			int minutes = seconds / 60;
			seconds %= 60;
			return "" + minutes + ":" + ((seconds < 10) ? "0" : "") + seconds;
		} else {
			return("N/A");
		}
	}

	public String getLength () {
		return("N/A");
	}

	public boolean acceptsType (String type) {
		return type.equals("mod") || type.equals(".xm") || type.equals(".it") || type.equals("s3m");
	}

	public Song getSong () {
		return song;
	}

	public void run () {
		byte[] b, lsl;
		int i, j, k, lsll;
		boolean cont = true;

		b = new byte[1024];
		statusLine = new byte[10];
		lsl = new byte[80];
		lsll = 0;

		for (i = 0; i < 10; ++i) {
			statusLine[i] = 32;
		}

		while (cont) {
			try {
				i = bis.read(b, 0, 1024);
				if (i > 0) {
					for (j = 0; j < i; ++j) {
						if (b[j] == 10 || b[j] == 13) {
							//							System.err.println(new String(lsl, 0, lsll));
							if (lsll > 5) {
								if (lsl[0] == 'P' && lsl[1] == ':') {
									for (k = 0; k < 10; ++k) {
										statusLine[k] = 32;
									}
									for (k = 2; k < 12; ++k) {
										statusLine[k-2] = lsl[k];
									}
								}
							}
							if (lsll > 8) {
								if (lsl[0] == 'T' && lsl[1] == 'i' && lsl[2] == 't' && lsl[3] == 'l' &&
									lsl[4] == 'e' && lsl[5] == ':') {
									title = new String(lsl, 7, lsll - 7);
								}
							}
							if (lsll == 61) {
								if (lsl[0] == 'C' && lsl[1] == 'o' && lsl[2] == 'u' && lsl[3] == 'l' &&
									lsl[4] == 'd') {
									status = "Retrying";
								}
							}
							lsll = 0;
						} else {
							lsl[lsll++] = b[j];
						}
					}
				} else if (i == -1) {
					bis.close();
					bis = null;
					cont = false;
				}
			} catch (Exception e) {
				bis = null;
				cont = false;
			}
		}
	}
}

==> musiccontrol-0.3.tar.gz: musiccontrol-0.3/src/dk/borderworlds/ModPlayer.java <==
// $Id: ModPlayer.java,v 1.12 2005/02/24 17:52:57 xi Exp $

/******************************************************************************
*    MusicControl - An advanced music player
*    Copyright (C) 2002-2005  Christian Laursen <xi@borderworlds.dk>
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

package dk.borderworlds;

import java.io.*;
import java.util.regex.*;

public class ModPlayer extends MusicPlayer implements Runnable {
	private Song song;
	private Process proc;
	private BufferedInputStream bis;
	String name;
	private Thread inputreader;
	private String status;
	private String statusLine;
	private String title, filename;
	private int pausePosition;
	private MusicConfiguration configuration;
	private boolean paused;
	private MusicPipe pipe;

	public ModPlayer (MusicConfiguration configuration) {
		song = null;
		proc = null;
		this.configuration = configuration;
		paused = false;
	}

	public void playSong (Song song) {
		this.song = song;
		playSong(0);
	}

	public void playSong (int offset) {
		if (offset == 0) {
			status = "Starting";
		} else {
			status = "Resuming";
		}
		title = song.getFilename();
		filename = song.getFilename();
		Runtime rt = Runtime.getRuntime();
		if (proc != null) {
			proc.destroy();
			while (bis != null) {
				try {
					Thread.sleep(300);
				} catch (Exception e) {
				}
			}
			try {
				Thread.sleep(300);
			} catch (Exception e) {
			}
			proc = null;
		}
		try {
			String device = configuration.getDevice();
			int args = (pausePosition == 0) ? 2 : 4;
			if (device.equals("STDOUT")) {
				args += 1;
			}
			String[] cmdarray = new String[args];
			cmdarray[0] = "simplemod";
			if (device.equals("STDOUT")) {
				cmdarray[1] = "-r";
			}
			if (pausePosition != 0) {
				cmdarray[args-3] = "-k";
				cmdarray[args-2] = "" + pausePosition;
			}
			cmdarray[args - 1] = song.getDirectory() + File.separator + song.getFilename();
			proc = rt.exec(cmdarray);
			InputStream is = proc.getErrorStream();
			pipe = new MusicPipe(proc.getInputStream());
			bis = new BufferedInputStream(is);
			inputreader = new Thread (this);
			inputreader.start();
			status = "Playing";
		} catch (Exception e) {
			System.err.println(e);
		}
	}

	public void pauseSong () {
		if (statusLine != null) {
			Pattern p = Pattern.compile(",(\\d+)$");
			Matcher m = p.matcher(statusLine);
			if (m.find()) {
				pausePosition = Integer.parseInt(m.group(1));
			} else {
				pausePosition = 0;
			}
		} else {
			pausePosition = 0;
		}

		paused = true;

		if (proc != null) {
			proc.destroy();
			while (bis != null) {
				try {
					Thread.sleep(300);
				} catch (Exception e) {
				}
			}
			proc = null;
		}
	}

	public void resumeSong () {
		if (paused) {
			playSong(pausePosition);
			pausePosition = 0;
			paused = false;
		}
	}

	public void stopSong () {
		if (proc != null) {
			proc.destroy();
			while (bis != null) {
				try {
					Thread.sleep(300);
				} catch (Exception e) {
				}
			}
			proc = null;
			song = null;
		}
		try {
			Thread.sleep(300);
		} catch (Exception e) {
		}
	}

	public String getStatus () {
		if (status.equals("Retrying")) {
			playSong(0);
			return "Retrying";
		}
		if (status.equals("Playing") && bis == null) {
			if (!paused) {
				status = "Stopped";
				song = null;
			} else {
				status = "Paused";
			}
		}
		return status;
	}

	public String getFilename () {
		if (song == null) {
			return "(none)";
		} else {
			return filename;
		}
	}

	public String getTitle () {
		if (song == null) {
			return "(none)";
		} else {
			return title;
		}
	}

	public String getPosition () {
		if (statusLine != null) {
			Pattern p = Pattern.compile("^\\s*(\\d+),");
			Matcher m = p.matcher(statusLine);
			if (m.find()) {
				int seconds = Integer.parseInt(m.group(1));
				int minutes = seconds / 60;
				seconds %= 60;
				return "" + minutes + ":" + ((seconds < 10) ? "0" : "") + seconds;
			} else {
				return "N/A";
			}
		} else {
			return("N/A");
		}
	}

	public String getLength () {
		return("N/A");
	}

	public boolean acceptsType (String type) {
		return type.equals("stm") || type.equals("mod") || type.equals(".xm") || type.equals(".it") || type.equals("s3m");
	}

	public Song getSong () {
		return song;
	}

	public void run () {
		byte[] b, lsl;
		int i, j, k, lsll;
		boolean cont = true;

		b = new byte[1024];
		statusLine = "";
		lsl = new byte[80];
		lsll = 0;

		Pattern titlePattern = Pattern.compile("^Title: (.*)$");
		Pattern timePattern = Pattern.compile("^P:(\\s*\\d+,\\d+)\\s");
		Pattern errorPattern = Pattern.compile("^Could not initialize sound");
		Matcher m;

		while (cont) {
			try {
				i = bis.read(b, 0, 1024);
				if (i > 0) {
					for (j = 0; j < i; ++j) {
						if (b[j] == 10 || b[j] == 13) {
							//							System.err.println(new String(lsl, 0, lsll));
							String line = new String(lsl, 0, lsll);

							m = timePattern.matcher(line);
							if (m.find()) {
								statusLine = m.group(1);
							}

							m = titlePattern.matcher(line);
							if (m.matches()) {
								title = m.group(1);
							}

							m = errorPattern.matcher(line);
							if (m.find()) {
								status = "Retrying";
							}

							lsll = 0;
						} else {
							lsl[lsll++] = b[j];
						}
					}
				} else if (i == -1) {
					bis.close();
					bis = null;
					cont = false;
				}
			} catch (Exception e) {
				bis = null;
				cont = false;
			}
		}
	}
}
